/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla browser.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Bill Haneman (bill.haneman@sun.com)
 *   Paul Sandoz (paul.sandoz@sun.com)
 *   John Gaunt (jgaunt@netscape.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"
#include "nsIAccessibleHyperLink.idl"

/**
 * A cross-platform interface that deals with text which contains hyperlinks.
 * Each link is an embedded object representing exactly 1 character within
 * the hypertext.
 *
 * Current implementation assumes every embedded object is a link.
 */

[scriptable, uuid(b33684e2-090c-4e1d-a3d9-f4b46f4237b9)]
interface nsIAccessibleHyperText : nsISupports
{
  /**
   * Return the number of links contained within this hypertext object.
   */
  readonly attribute long linkCount;

  /**
   * Return link accessible at the given index.
   *
   * @param index  [in] 0-based index of the link that is to be retrieved
   *
   * @return       link accessible or null if there is no link at that index
   */
  nsIAccessibleHyperLink getLinkAt(in long index);

  /**
   * Return index of the given link.
   *
   * @param link  [in] link accessible the index is requested for
   *
   * @return      index of the given link or null if there's no link within
   *                hypertext accessible
   */
  long getLinkIndex(in nsIAccessibleHyperLink link);

  /*
   * Return link index at the given offset within hypertext accessible.
   *
   * @param offset  [in] the 0-based character index
   *
   * @return        0-based link's index or -1 if no link is present at that
   *                  offset
   */
  long getLinkIndexAtOffset(in long offset);
};
