/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is the Mozilla Corporation
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Neil Deakin (enndeakin@sympatico.ca)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== datetimepicker.css =============================================
  == Styles used by the XUL datepicker and timepicker elements.
  ======================================================================= */
  
@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
@namespace html url("http://www.w3.org/1999/xhtml");

datepicker, timepicker {
  margin: 2px 4px;
  padding: 0;
  border: none;
  background: none;
  cursor: default;
}

.datetimepicker-input-box {
  -moz-appearance: textfield;
  cursor: text;
  -moz-margin-end: 2px;
  border: 2px solid;
  -moz-border-top-colors: ThreeDShadow ThreeDDarkShadow;
  -moz-border-right-colors: ThreeDHighlight ThreeDLightShadow;
  -moz-border-bottom-colors: ThreeDHighlight ThreeDLightShadow;
  -moz-border-left-colors: ThreeDShadow ThreeDDarkShadow;
  padding: 2px 0 3px 0;
  -moz-padding-start: 4px;
  -moz-padding-end: 2px;
  background-color: -moz-Field;
  color: -moz-FieldText;
}

.datetimepicker-input-subbox {
  width: 1.6em;
}

html|*.datetimepicker-input {
  text-align: end;
}

.datetimepicker-separator {
  margin: 0 !important;
}

.datetimepicker-year {
  width: 3.2em;
}

datepicker[readonly="true"],
timepicker[readonly="true"] {
  background-color: -moz-Dialog;
  color: -moz-DialogText;
}

datepicker[disabled="true"],
timepicker[disabled="true"] {
  cursor: default;
  background-color: -moz-Dialog;
  color: GrayText;
} 

.datepicker-mainbox {
  margin: 2px 4px;
  border: 2px solid;
  -moz-border-top-colors: ThreeDShadow ThreeDDarkShadow;
  -moz-border-right-colors: ThreeDHighlight ThreeDLightShadow;
  -moz-border-bottom-colors: ThreeDHighlight ThreeDLightShadow;
  -moz-border-left-colors: ThreeDShadow ThreeDDarkShadow;
  background-color: -moz-Field;
  color: -moz-FieldText;
}

.datepicker-popupgrid > .datepicker-mainbox {
  margin: 0;
  border: none;
}

.datepicker-gridlabel, .datepicker-weeklabel {
  text-align: center;
}

.datepicker-gridlabel[today="true"] {
  background-color: darkgrey;
  color: white;
}

.datepicker-gridlabel[selected="true"] {
  background-color: Highlight;
  color: HighlightText;
}

.datepicker-button {
  -moz-appearance: none;
  min-width: 8px;
  padding: 0px;
}

.datepicker-previous {
  list-style-image: url("chrome://global/skin/arrow/arrow-lft.gif");
}

.datepicker-next {
  list-style-image: url("chrome://global/skin/arrow/arrow-rit.gif");
}

.datepicker-previous:hover {
  list-style-image: url("chrome://global/skin/arrow/arrow-lft-hov.gif");
}

.datepicker-next:hover {
  list-style-image: url("chrome://global/skin/arrow/arrow-rit-hov.gif");
}

.datepicker-previous[disabled="true"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-lft-dis.gif");
}

.datepicker-next[disabled="true"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-rit-dis.gif");
}

.datepicker-previous:-moz-locale-dir(rtl) {
  list-style-image: url("chrome://global/skin/arrow/arrow-rit.gif");
}

.datepicker-next:-moz-locale-dir(rtl) {
  list-style-image: url("chrome://global/skin/arrow/arrow-lft.gif");
}

.datepicker-previous:-moz-locale-dir(rtl) {
  list-style-image: url("chrome://global/skin/arrow/arrow-rit-hov.gif");
}

.datepicker-next:-moz-locale-dir(rtl):hover {
  list-style-image: url("chrome://global/skin/arrow/arrow-lft-hov.gif");
}

.datepicker-previous[disabled="true"]:-moz-locale-dir(rtl) {
  list-style-image: url("chrome://global/skin/arrow/arrow-rit-dis.gif");
}

.datepicker-next[disabled="true"]:-moz-locale-dir(rtl) {
  list-style-image: url("chrome://global/skin/arrow/arrow-lft-dis.gif");
}
