/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* Stock icons for the menu bar items */
menuitem:not([type]) {
  -moz-binding: url("chrome://global/content/bindings/menu.xml#menuitem-iconic");
}

#menu_savePage {
  list-style-image: url("moz-icon://stock/gtk-save-as?size=menu");
}

#menu_printPreview {
  list-style-image: url("moz-icon://stock/gtk-print-preview?size=menu");
}

#menu_print {
  list-style-image: url("moz-icon://stock/gtk-print?size=menu");
}

#menu_close {
  list-style-image: url("moz-icon://stock/gtk-close?size=menu");
}

#menu_undo {
  list-style-image: url("moz-icon://stock/gtk-undo?size=menu");
}

#menu_undo[disabled] {
  list-style-image: url("moz-icon://stock/gtk-undo?size=menu&state=disabled");
}

#menu_redo {
  list-style-image: url("moz-icon://stock/gtk-redo?size=menu");
}

#menu_redo[disabled] {
  list-style-image: url("moz-icon://stock/gtk-redo?size=menu&state=disabled");
}

#menu_cut {
  list-style-image: url("moz-icon://stock/gtk-cut?size=menu");
}

#menu_cut[disabled] {
  list-style-image: url("moz-icon://stock/gtk-cut?size=menu&state=disabled");
}

#cMenu_copy,
#menu_copy {
  list-style-image: url("moz-icon://stock/gtk-copy?size=menu");
}

#menu_copy[disabled] {
  list-style-image: url("moz-icon://stock/gtk-copy?size=menu&state=disabled");
}

#menu_paste {
  list-style-image: url("moz-icon://stock/gtk-paste?size=menu");
}

#menu_paste[disabled] {
  list-style-image: url("moz-icon://stock/gtk-paste?size=menu&state=disabled");
}

#menu_delete {
  list-style-image: url("moz-icon://stock/gtk-delete?size=menu");
}

#menu_delete[disabled] {
  list-style-image: url("moz-icon://stock/gtk-delete?size=menu&state=disabled");
}

#cMenu_selectAll,
#menu_selectAll {
  list-style-image: url("moz-icon://stock/gtk-select-all?size=menu");
}

#menu_find {
  list-style-image: url("moz-icon://stock/gtk-find?size=menu");
}

#menu_find[disabled] {
  list-style-image: url("moz-icon://stock/gtk-find?size=menu&state=disabled");
}

#menu_goToLine {
  list-style-image: url("moz-icon://stock/gtk-jump-to?size=menu");
}

#menu_goToLine[disabled] {
  list-style-image: url("moz-icon://stock/gtk-jump-to?size=menu&state=disabled");
}

#menu_reload {
  list-style-image: url("moz-icon://stock/gtk-refresh?size=menu");
}

#menu_reload[disabled] {
  list-style-image: url("moz-icon://stock/gtk-refresh?size=menu&state=disabled");
}

#menu_textEnlarge {
  list-style-image: url("moz-icon://stock/gtk-zoom-in?size=menu");
}

#menu_textReduce {
  list-style-image: url("moz-icon://stock/gtk-zoom-out?size=menu");
}

#menu_textReset {
  list-style-image: url("moz-icon://stock/gtk-zoom-100?size=menu");
}

#menu_openHelp {
  list-style-image: url("moz-icon://stock/gtk-help?size=menu");
}

#aboutName {
  list-style-image: url("moz-icon://stock/gtk-about?size=menu");
}
