
.console-box {
  -moz-binding: url("chrome://global/content/consoleBindings.xml#console-box");
  overflow: auto;
}

.console-rows {
  -moz-user-focus: normal;
}

.console-row[type="error"],
.console-row[type="warning"] {
  -moz-binding: url("chrome://global/content/consoleBindings.xml#error");
}

.console-row[type="message"] {
  -moz-binding: url("chrome://global/content/consoleBindings.xml#message");
}

.console-msg-text,
.console-error-msg {
  white-space: pre-wrap;
}

.console-error-source {
  -moz-binding: url("chrome://global/content/consoleBindings.xml#console-error-source");
}

.console-dots {
  width: 1px;
}

/* :::::::::: hiding and showing of rows for each mode :::::::::: */

.console-box[mode="Warnings"] > .console-box-internal > .console-rows 
    > .console-row[type="error"],
.console-box[mode="Messages"] > .console-box-internal > .console-rows 
    > .console-row[type="error"]
{
  display: none;
}

.console-box[mode="Errors"] > .console-box-internal > .console-rows 
    > .console-row[type="warning"],
.console-box[mode="Messages"] > .console-box-internal > .console-rows 
    > .console-row[type="warning"]
{
  display: none;
}

.console-box[mode="Errors"] > .console-box-internal > .console-rows 
    > .console-row[type="message"],
.console-box[mode="Warnings"] > .console-box-internal > .console-rows 
    > .console-row[type="message"]
{
  display: none;
}

/* If line number is 0, hide the line number section */
.lineNumberRow[line="0"] {
  display: none;
}

#TextboxEval {
  direction: ltr;
}
