#xpinstallheader {
  margin-bottom: 2em;
}

#itemList {
  -moz-appearance: listbox;
  margin: 3px 4px 10px 4px;
  height: 14em;
  border: 2px solid;
  -moz-border-top-colors: ThreeDShadow ThreeDDarkShadow;
  -moz-border-right-colors: ThreeDHighlight ThreeDLightShadow;
  -moz-border-bottom-colors: ThreeDHighlight ThreeDLightShadow;
  -moz-border-left-colors: ThreeDShadow ThreeDDarkShadow;
  background-color: -moz-Field;
  color: -moz-FieldText;
}

#itemWarningIntro {
  -moz-margin-start: 8px;
}

#dialogContentBox {
  padding: 5px;
}

installitem {
  padding-top: 5px;
  padding-bottom: 5px;
  -moz-padding-start: 5px;
  -moz-padding-end: 0;
  border-bottom: 1px dotted #C0C0C0;
  margin-bottom: 5px;
}

.alert-icon {
  -moz-margin-end: 20px;
}

.warning {
  font-weight: bold;
  font-size: 1.25em;
  margin-bottom: 1em;
}

.xpinstallIconContainer {
  width: 32px;
  height: 32px;
  -moz-margin-end: 5px;
}

.xpinstallItemName {
  font-weight: bold;
}

.xpinstallItemSigned {
  font-style: italic;
  font-size: 0.9em;
}

.xpinstallItemURL {
  -moz-appearance: none;
  border: none;
  padding: 0;
  background-color: -moz-Field;
  color: -moz-FieldText;
  margin-top: 1px;
  margin-bottom: 1px;
  -moz-margin-start: 6px;
  -moz-margin-end: 5px;
}

.xpinstallItemIcon {
  max-width: 32px;
  max-height: 32px;
  list-style-image: url("chrome://mozapps/skin/extensions/extensionGeneric.png");
}

installitem[type="theme"] .xpinstallItemIcon {
  list-style-image: url("chrome://mozapps/skin/extensions/themeGeneric.png");
}

installitem[type="locale"] .xpinstallItemIcon {
  list-style-image: url("chrome://mozapps/skin/extensions/localeGeneric.png");
}

installitem[type="plugin"] .xpinstallItemIcon {
  list-style-image: url("chrome://mozapps/skin/plugins/pluginGeneric.png");
}
